@@ -47,12 +47,15 @@
 #include "xf86cmap.h"
 #include "shadowfb.h"
 #include "vgaHW.h"
-#include "xaa.h"
 #include "compiler.h"
 
+#ifdef HAVE_XAA_H
+#include "xaa.h"
+#endif
+
 #include "voodoo.h"
 
-#include <X11/extensions/xf86dgastr.h>
+#include <X11/extensions/xf86dgaproto.h>
 
 #include "opaque.h"
 #ifdef HAVE_XEXTPROTO_71
@@ -67,7 +70,7 @@
 
 #include <unistd.h>
 
-
+#ifdef HAVE_XAA_H
 #if 0
 static void VoodooReadWriteBank(ScreenPtr pScreen, int bank);
 #endif
@@ -82,7 +85,7 @@ static Bool VoodooSetupForCPUToScreenTexture(ScrnInfoPtr pScrn, int op,
 static void VoodooSubsequentCPUToScreenTexture(ScrnInfoPtr pScrn,
 	int dstx, int dsty, int srcx, int srcy, int width, int height);
 
-
+#endif
 static int debug = 0;
 
 /*
@@ -811,7 +814,7 @@ void VoodooClear(VoodooPtr pVoo)
  
 void VoodooReadBank(ScreenPtr pScreen, int bank)
 {
-	ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	VoodooPtr pVoo = VoodooPTR(pScrn);
 	if(bank)
 	{
@@ -828,7 +831,7 @@ void VoodooReadBank(ScreenPtr pScreen, int bank)
 
 void VoodooWriteBank(ScreenPtr pScreen, int bank)
 {
-	ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	VoodooPtr pVoo = VoodooPTR(pScrn);
 	if(bank)
 	{
@@ -846,7 +849,7 @@ void VoodooWriteBank(ScreenPtr pScreen, int bank)
 #if 0
 static void VoodooReadWriteBank(ScreenPtr pScreen, int bank)
 {
-	ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	VoodooPtr pVoo = VoodooPTR(pScrn);
 	if(bank)
 	{
@@ -920,6 +923,7 @@ void VoodooSync(ScrnInfoPtr pScrn)
 	mmio32_w(pVoo, 0x10C, 0);	/* Maybe flag this */
 }
 
+#ifdef HAVE_XAA_H
 static void Voodoo2Setup2D(VoodooPtr pVoo)
 {
 	wait_idle(pVoo);
@@ -1385,11 +1389,13 @@ static void VoodooSubsequentCPUToScreenTexture(ScrnInfoPtr pScrn,
 CARD32 VoodooAlphaTextureFormats[2] = {PICT_a8, 0};
 CARD32 VoodooTextureFormats[3] = {PICT_a8r8g8b8, PICT_x8r8g8b8, 0};
 
+#endif
 #endif
 
 void Voodoo2XAAInit(ScreenPtr pScreen)
 {
-	ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+#ifdef HAVE_XAA_H
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	VoodooPtr pVoo = VoodooPTR(pScrn);
 	XAAInfoRecPtr pAccel = XAACreateInfoRec();
 	BoxRec cacheArea;
@@ -1472,4 +1478,5 @@ void Voodoo2XAAInit(ScreenPtr pScreen)
 		ErrorF("Unable to set up acceleration.\n");
 		
 	Voodoo2DisableClipping(pScrn);
+#endif
 }
